/*
    FxFNav (Fenix File Navigator) is a file manager for GP2x Wiz game console.
    Copyright (C) 2009  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
GLOBAL
  string commander_cmd = "/tmp/comm.cmd";
  string commander_res = "/tmp/comm.res";
  string commander_command = 0;
  int res_fd = 0;
END

FUNCTION COMMANDER_INIT(string cmd_pipe, string res_pipe)
BEGIN
  commander_cmd = cmd_pipe;
  commander_res = res_pipe;
END;


FUNCTION COMMANDER_EXEC(string command)
LOCAL
  int fd;
END
BEGIN
  say("COMMANDER_EXEC("+command+")");
  fd = fopen(commander_cmd, O_WRITE);
  IF(!fd)
    RETURN -1;
  END;
  fputs(fd, command);
  commander_command = command;
  fclose(fd);
  res_fd = fopen(commander_res, O_READ);
  RETURN 0;
END;

FUNCTION COMMANDER_GET_RES(string * one_line)
LOCAL
  int fd;
END
BEGIN
  //say("COMMANDER_GET_RES( "+one_line+" / "+res_fd+" )");
  
  IF(!res_fd)
    *one_line = "<ERROR>";
    say("No command running");
    RETURN -1;
  END;
  
  *one_line = fgets(res_fd); 
  
  IF(*one_line=="OK" || *one_line=="ERROR")
    fclose(res_fd);
    RETURN -2;
  END;
  //say("END "+*one_line);
  RETURN 0;
END;

PROCESS  FILE_INFO()
PUBLIC
    string name = "";
    string ext = "";
    int fsize = 0;
    int f_dir;
    int f_hidden;
    int f_exe;
    int f_link;
    string linkto = "";
    string opts = "";
END
BEGIN
    alpha = 0;
    SIGNAL(id, S_SLEEP);
    FRAME RAND(100,10000);
END;

FUNCTION CPY_FILE_INFO( int _xto,  int _frm)
PRIVATE
  FILE_INFO xto;
  FILE_INFO frm;
END
BEGIN
  xto = _xto;
  frm = _frm;
  xto.name     =frm.name;
  xto.ext      =frm.ext;
  xto.fsize    =frm.fsize;
  xto.f_dir    =frm.f_dir;
  xto.f_hidden =frm.f_hidden;
  xto.f_exe    =frm.f_exe;
  xto.f_link   =frm.f_link;
  xto.linkto   =frm.linkto;
  xto.opts     =frm.opts;  
END;

FUNCTION LS2FILEINFO(string path, string ls, int _info)
LOCAL
  int p, p1, i, l;
  string ch, ich, link;
  FILE_INFO info;
END
BEGIN
  info = _info;
  //info.opts = "";
  info.f_link = info.f_exe = info.f_hidden = info.f_dir = 0;
  
  IF( (ch = glob(path+ls) ) != "" )
    info.name  = fileinfo.name;  
    info.f_dir = fileinfo.directory;
    info.fsize  = fileinfo.size;
  ELSE
    info.name  = "ERROR";  
    info.f_dir = "ERROR";
    info.fsize  = 0;
  END;
  
  //IF( ich=="/" ) info.f_dir = 1; END;
  //IF( find(ch,'*')<0 ) info.f_exe = 1; END;
  
  /*
  p=0;
  FOR(i=10;i<l;i++)
    IF(ls[i]==' ' && ls[i-1]!=' ') 
      p++; 
      IF( p==4 ) p1=i; END; 
      IF( p==5 ) 
	info.size = atoi(substr(ls,p1,i-p1));
      END; 
      IF(p==8) info.name=substr(ls,i+1,l-i-1); BREAK; END; 
    END;
  END;
  */  
  
  ///info.name = UNESCAPELS(info.name, &info.linkto);
//  link = info.linkto;
/*  UNESCAPELS(&info.name, &info.linkto);
  IF(info.linkto != "")
    info.f_link = 1;
    say("link");
  END;*/
  
END;

FUNCTION string ESCAPESHELLARG(string ls)
LOCAL
  string escape[]=("'", "");
  int i,j,l;
  string obuf;
  string tmp;
END
BEGIN
  tmp = ls;
  FOR( i=0; escape[i]!=""; i++)
    obuf = "";
    l = len(tmp);
    
    FOR(j=0;j<l;j++)
      IF( tmp[j]==escape[i] )
	obuf += "\"+tmp[j];
      ELSE
	obuf += tmp[j];
      END;
    END;
    tmp = obuf;

  END;
  RETURN "'" + tmp + "'";
END;

// parsing ls -QF output
FUNCTION UNESCAPELS(string *argx, string *linkto)
LOCAL
  int i,l;
  string obuf, arg;
  string stop, lnk="";
END
BEGIN
  arg = *argx;
  obuf = "";
  l = len(arg);
  stop = arg[0];
  
  FOR(i=1; i<l-1; i++)
    IF(arg[i]==stop) BREAK; END;
    IF(arg[i]=='\') 
      i++;
      obuf += arg[i];
      CONTINUE; 
    END;
    obuf += arg[i];
  END;

  i+=2;
  /// it's a symlink
  IF(i<l)
    // pass " -> "
    FOR(; arg[i] != stop && i<l-1; i++) END;
    FOR(i++; i<l-1; i++)
      IF(arg[i]==stop) BREAK; END;
      IF(arg[i]=='\') 
	i++;
	lnk += arg[i];
	CONTINUE; 
      END;
      lnk += arg[i];
    END;
  END;
  
  *linkto = lnk;  
  *argx = obuf;
END;

GLOBAL
  coomander_task_pid = 0;
END
FUNCTION COMMANDER_TASK(string command, int sync)
LOCAL
  int fd;
END
BEGIN
  say("COMMANDER_TASK("+command+")");
  coomander_task_pid++;
  
  fd = fopen(commander_cmd, O_WRITE);
  IF(!fd)
    RETURN -1;
  END;
  fputs(fd, "TASK");
  fclose(fd);
  
  // 1 sec pause lets commander.sh to read next commad
  FRAME Config.fps*100;

  fd = fopen(commander_cmd, O_WRITE);
  IF(!fd)
    RETURN -1;
  END;
  fputs(fd, "/tmp/fxfnav-task."+coomander_task_pid);
  fclose(fd);
  
  // 1 sec pause lets commander.sh to read next commad
  FRAME Config.fps*100;

  fd = fopen(commander_cmd, O_WRITE);
  IF(!fd)
    RETURN -1;
  END;
  fputs(fd, command);
  fclose(fd);
  
  // 1 sec pause lets commander.sh to read next commad
  FRAME Config.fps*100;

  IF(!sync)
    // bg task
    RETURN coomander_task_pid;
  ELSE
    // witing for results
    say("waiting");
    LOOP
      FRAME;
      say("pip");
      fd = fopen("/tmp/fxfnav-task."+coomander_task_pid, O_READ);
      IF(!fd)
	BREAK;
      END;      
    END;
    say("task done");
    RETURN coomander_task_pid;
  END;
END;

FUNCTION CHDIR_AND_EXEC(string dir, string command)
LOCAL
  int f;
END  
BEGIN
  f = fopen(Config.cd_on_exit_file, O_WRITE);
  fputs(f,dir);
  fclose(f);
  f = fopen(Config.exec_on_exit_file, O_WRITE);
  fputs(f,command);
  fclose(f);
END;