/*
    FxFNav (Fenix File Navigator) is a file manager for GP2x Wiz game console.
    Copyright (C) 2009  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
GLOBAL
  Manipulator_Line Last_Manipulator =-1;
  int mp_colors[4];
  string mp_messages[4];
END

CONST
  MP_MODE_COPY   = 1;
  MP_MODE_MOVE   = 2;  
  MP_MODE_DELETE = 3;
END

PROCESS Manipulator_Line(string _full_path, string _display_name, int protected)
PUBLIC
  string full_path, destination="";
  int tox, toy, toangle, ww, hh;
  int as,ds;
  int mode, xmode, doit;
END
BEGIN
  //SAY("Manipulator");
  full_path = _full_path;

  IF(EXISTS(Last_Manipulator))
    IF(_full_path == Last_Manipulator.full_path)
      IF(protected) 
	Last_Manipulator.mode = MP_MODE_COPY;
	RETURN;
      END;

      IF(Last_Manipulator.mode < MP_MODE_DELETE )
	Last_Manipulator.mode ++;
      ELSE
	Last_Manipulator.mode = MP_MODE_COPY;
      END;

      RETURN;
    ELSE
      SIGNAL(Last_Manipulator, S_KILL);
      //FRAME;
    END;
  END;
  Last_Manipulator = id;

  set_text_color(mp_colors[MP_MODE_COPY]); 
  
  xmode = mode = MP_MODE_COPY;

  graph = write_truettf(0, 0, Config.ttt_font, 0, 0, TT_NEW_MAP | TT_ENC_UTF8, _display_name);
  
  ww  = Graphic_info(file,graph,G_WIDTH );
  hh  = Graphic_info(file,graph,G_HEIGHT );
  id.angle = 300;
  tox= x;
  toy= y;
  toangle=angle;
  
  as = 5000;
  ds = Config.fps / 2.5;
  doit = 0;

  LOOP

    IF(x < tox) x+=ds; END;
    IF(x > tox) x-=ds; END;
    IF(y < toy) y+=ds; END;
    IF(y > toy) y-=ds; END;

    IF(angle < toangle) angle+=as; END;
    IF(angle > toangle) angle-=as; END;
    
    IF(iabs(x-tox)<=ds) x=tox; END;
    IF(iabs(y-toy)<=ds) y=toy; END;
    IF(iabs(angle-toangle)<=as) angle=toangle; END;

    IF(xmode!=mode)
      set_text_color(mp_colors[mode]); 
      IF(x == tox && y == toy) x -= 3*ds; END;
      write_truettf(file, graph, Config.ttt_font, 0, 0, TT_ENC_UTF8, _display_name);      
    END;
    
    xmode = mode;
    FRAME;
    
    IF(EXISTS(doit))
      Operation(full_path, destination, mode, doit);
      doit = 0;
    END;
  END;

ONEXIT
  unload_map( file, graph );
END;

PROCESS Operation(string src, string dst, int operation, int unsleep)
LOCAL
  int rkey;
END
BEGIN
  Last_Manipulator.tox = Config.SW / 2;
  Last_Manipulator.toy = Config.TOPPAD +1 + Last_Manipulator.hh / 2;
  Last_Manipulator.toangle = 0;
  
  set_text_color(mp_colors[operation]); 
  
  DLG_OPERATION(id, mp_messages[operation] ,&rkey);
  FRAME;

  src = escapeshellarg(src);
  dst = escapeshellarg(dst);
  
  IF( rkey == _WIZ_L )
    SIGNAL(Last_Manipulator, S_KILL);
    graph = Config.map_clock;
    x = Config.SW/2;
    y = Config.SH/2;
    FRAME 0;
    IF(operation == MP_MODE_COPY )
      COMMANDER_TASK("cp -r "+src+" "+dst,true);
    ELSEIF(operation == MP_MODE_MOVE)
      COMMANDER_TASK("mv "+src+" "+dst,true);
    ELSEIF(operation == MP_MODE_DELETE)
      COMMANDER_TASK("rm -rf "+src,true);
    END;
  ELSE
    Last_Manipulator.tox = Config.SW - 1 - Last_Manipulator.hh / 2;
    Last_Manipulator.toy = Config.TOPPAD + (Config.SH/2 - Config.TOPPAD);
    Last_Manipulator.toangle = 90000;
  END;

  SIGNAL(unsleep, S_WAKEUP);
  FRAME;
END;
