/*
    FxFNav (Fenix File Navigator) is a file manager for GP2x Wiz game console.
    Copyright (C) 2009  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

include "image.fh";
include "truettf.fh";

#define PREVIEW_MODE_PREVIEW 1
#define PREVIEW_MODE_VIEW 2

GLOBAL
  Prev_Nav = 0;
END

CONST
  preview_h = 110;
  preview_w = 160;
END

PROCESS Preview(string fullpath, int _info)
LOCAL
  Preview others;
  FILE_INFO info;
  string infostr, ext;
  int map, w, h, k;
  int kpreview, kview , viewable;
  int mode;
  int gpv, gv;
END
BEGIN
  //ReturN;
  info = _info;
  say("PREVIEW: "+fullpath+" @ "+info.name);
  map = -1;
  ext = info.ext;
  graph = gpv = new_map(preview_w, preview_h*1.5, 16);
  gv = new_map(Config.SW, Config.SH, 16);

  mode  = PREVIEW_MODE_PREVIEW;
  viewable = 0;
  
  IF(EXISTS(Prev_Nav))
    SIGNAL(Prev_Nav, S_KILL);
  END;
  
  Prev_Nav = id;
  
  IF(info.f_dir)
    infostr = _("Folder");
  ELSE
    infostr = _("File");
  END

  IF(
      !info.f_dir &&
      (ext == "png" || ext == "jpg" || ext == "jpeg" || ext == "gif" || ext == "bmp" || ext == "pcx" || ext == "lbm" || ext == "tif" || ext == "pnm")
    )
    infostr = _("Image");
    viewable = 1;
    IF(!Config.use_preview)
	infostr += "["+_("preview is off")+"]";
    ELSEIF(info.fsize > Config.max_preview_size)
	infostr += "["+_("too big")+"]";
    ELSE
    
      map = load_image(fullpath);
      
      IF(map>0)
	w  = Graphic_info(file,map,G_WIDTH );
	h  = Graphic_info(file,map,G_HEIGHT );

	kpreview = preview_w*100/w;
	IF( kpreview > preview_h*100/h) kpreview = preview_h*100/h; END;
	IF(kpreview>100) kpreview=100; END;

	kview = Config.SW*100/w;
	IF( kview > Config.SH*100/h) kview = (Config.SH )*100/h; END;
	//IF(kview>100) kview=100; END;

	map_xput ( file , gpv , map , preview_w/2 , preview_h/2 , 0 , kpreview , 0 );
	map_xput ( file , gv , map , Config.SW/2 , Config.SH/2, 0 , kview , 0 );
	infostr += " ["+w+"x"+h+"]";
      ELSE
	viewable=0;
      END;
      
    END;
  ELSE
  //
  END;
  
  write_truettf(file, graph, Config.ttt_font, 0, preview_h+1, TT_ALIGN_H_LEFT | TT_ALIGN_V_TOP, infostr);    
  write_truettf(file, graph, Config.ttt_font, 0, preview_h+15, TT_ALIGN_H_LEFT | TT_ALIGN_V_TOP, _("size: ")+info.fsize+_(" bytes"));
  x = Config.SW -10 - preview_w/2;
  y = Config.SH -10 - preview_h/2;
  z = 300;
  flags = B_TRANSLUCENT;

  LOOP
    IF(mode == PREVIEW_MODE_PREVIEW)
      IF(size>100) size-=9; ELSE size = 100; END;
      IF(x < Config.SW -10 - preview_w/2) x+=9; ELSE x = Config.SW -10 - preview_w/2; END;
      IF(y < Config.SH -10 - preview_h/2) y+=7; ELSE y = Config.SH -10 - preview_h/2; END;
    ELSE
      IF(size<100) size+=7; ELSE size = 100; END;
      IF(x > Config.SW/2) x-=7; ELSE x = Config.SW/2; END;
      IF(y > Config.SH/2) y-=6; ELSE y = Config.SH/2; END;
    END;

    k = mode;
    FRAME;
    IF(k == PREVIEW_MODE_PREVIEW && mode == PREVIEW_MODE_VIEW)
      IF(!viewable) mode = PREVIEW_MODE_PREVIEW; CONTINUE; END;
      IF(map<0)
	// force load view & preview on request
	map = load_image(fullpath);
	
	IF(map>0)
	  w  = Graphic_info(file,map,G_WIDTH );
	  h  = Graphic_info(file,map,G_HEIGHT );

	  kpreview = preview_w*100/w;
	  IF( kpreview > preview_h*100/h) kpreview = preview_h*100/h; END;
	  IF(kpreview>100) kpreview=100; END;

	  kview = Config.SW*100/w;
	  IF( kview > Config.SH*100/h) kview = (Config.SH )*100/h; END;

	  map_xput ( file , gpv , map , preview_w/2 , preview_h/2 , 0 , kpreview , 0 );
	  map_xput ( file , gv , map , Config.SW/2 , Config.SH/2, 0 , kview , 0 );
	  infostr += " ["+w+"x"+h+"]";
	ELSE
	  viewable = 0;
	  CONTINUE;
	END;
      END;
      z = 5;
      flags = 0;
      graph = gv;
      size = preview_w *100 / Config.SW ;
    END;
    IF(k == PREVIEW_MODE_VIEW && mode == PREVIEW_MODE_PREVIEW)
      z = 300;
      flags = B_TRANSLUCENT;
      graph = gpv;
      size = Config.SW*100 / preview_w ;
    END;

  END;
  
ONEXIT
  IF(map>0) unload_map(file,map); END;
  unload_map(file,gv);
  unload_map(file,gpv);
END;

/*
FUNCTION LoadImageForPreview(filename, int xw, int xh, int *amap, int *k)
LOCAL
 int w,h;
END;
BEGIN
  *amap = load_image(filename);
  w  = Graphic_info(file,*amap,G_WIDTH );
  h  = Graphic_info(file,*amap,G_HEIGHT );
  *k = xw*100/w;
  IF( *k > xh*100/h) *k = xh*100/h; END;
END;

*/